// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/KeypadConundrum.hpp>

#include <map>
#include <sstream>

#include <aoc/common/Point2.hpp>

KeypadConundrum::KeypadConundrum()
    : numericKeyboardRobot_{ { { 'A', { 0, 0 } }, { '0', { -1, 0 } }, { '1', { -2, -1 } }, { '2', { -1, -1 } },
          { '3', { 0, -1 } }, { '4', { -2, -2 } }, { '5', { -1, -2 } }, { '6', { 0, -2 } }, { '7', { -2, -3 } },
          { '8', { -1, -3 } }, { '9', { 0, -3 } } }, { -2, 0 } },
      directionalKeyboardRobot_{ { { 'A', { 0, 0 } }, { '<', { -2, 1 } }, { '>', { 0, 1 } }, { '^', { -1, 0 } },
          { 'v', { -1, 1 } } }, { -2, 0 } }
{
}

const std::string KeypadConundrum::getPuzzleName() const
{
    return "Keypad Conundrum";
}

const int KeypadConundrum::getPuzzleDay() const
{
    return 21;
}

void KeypadConundrum::processDataLine(const std::string& line)
{
    std::istringstream stream{ line };
    int64_t number;
    stream >> number;
    std::string inputKeys{ numericKeyboardRobot_.calcInputKeys(line) };
    for (size_t i = 0; i < 2; i++)
    {
        inputKeys = directionalKeyboardRobot_.calcInputKeys(inputKeys);
    }
    part1 += number * static_cast<int64_t>(inputKeys.size());
}

void KeypadConundrum::finish()
{
}

constexpr char KeypadConundrum::getStartPositionChar()
{
    return 'A';
}
